// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"

namespace Rivet {

  //differential xsection measurement in different variables
  class ATLAS_2015_I1394679 : public Analysis {
  public:

    /// Constructor
    ATLAS_2015_I1394679()
      : Analysis("ATLAS_2015_I1394679")
    {   
      setNeedsCrossSection(true);
    }


    /// @name Analysis methods
    //@

    /// Book histograms and initialise projections before the run
    void init() {

      //Initialise and register projections here
      const FinalState fs;
      addProjection(fs, "FinalState");
      FastJets fj04(fs, FastJets::ANTIKT, 0.4);
      fj04.useInvisibles();
      addProjection(fj04, "AntiKt4jets");

      // histograms
      // binning->taken directly from hepdata file
      _h["pt1"] = bookHisto1D(1, 1, 1);
      _h["pt2"] = bookHisto1D(2, 1, 1);
      _h["pt3"] = bookHisto1D(3, 1, 1);
      _h["pt4"] = bookHisto1D(4, 1, 1);
      _h["HT"]  = bookHisto1D(5, 1, 1);
      _h["M4j"] = bookHisto1D(6, 1, 1);
      //book histograms with different pt/m4j cuts
      for (int i_hist = 0; i_hist < 4; ++i_hist) {

        string label = "M2jratio_" + to_str(i_hist);
        _h[label] = bookHisto1D( 7 + i_hist, 1, 1);
        label = "dPhiMin2j_" + to_str(i_hist);
        _h[label] = bookHisto1D(11 + i_hist, 1, 1);
        label = "dPhiMin3j_" + to_str(i_hist);
        _h[label] = bookHisto1D(15 + i_hist, 1, 1);
        label = "dYMin2j_" + to_str(i_hist);
        _h[label] = bookHisto1D(19 + i_hist, 1, 1);
        label = "dYMin3j_" + to_str(i_hist);
        _h[label] = bookHisto1D(23 + i_hist, 1, 1);
        label = "dYMax2j_" + to_str(i_hist);
        _h[label] = bookHisto1D(27 + i_hist, 1, 1);

        for (unsigned int ygap = 0; ygap < 4; ++ygap) {
          label = "sumPtCent_" + to_str(ygap) + to_str(i_hist);
          _h[label] = bookHisto1D(31 + i_hist + ygap * 4, 1, 1);
        }

      }

    }

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    /// Perform the per-event analysis
    void analyze(const Event& event) {

      const double weight = event.weight();
      const Jets& alljetskt4 = applyProjection<FastJets>(event, "AntiKt4jets").jetsByPt(Cuts::pT > 60*GeV && Cuts::absrap < 2.8);

      //Event cuts
      //identify 4 leading jets with rap<2.8 and passing pt cuts
      int nJets = alljetskt4.size();
      if (nJets < 4)  vetoEvent;

      vector <FourMomentum> leadingJetskt4;
      FourMomentum jet1 = alljetskt4[0].momentum(); leadingJetskt4.push_back(jet1);
      FourMomentum jet2 = alljetskt4[1].momentum(); leadingJetskt4.push_back(jet2);
      FourMomentum jet3 = alljetskt4[2].momentum(); leadingJetskt4.push_back(jet3);
      FourMomentum jet4 = alljetskt4[3].momentum(); leadingJetskt4.push_back(jet4);

      //dR cut
      float dRcut = 0.65;
      float drmin = 9999;
      for (int ijet = 0; ijet < 4; ++ijet) {
        for (int jjet = ijet + 1; jjet < 4; ++jjet) {
          float myDR = deltaR(alljetskt4[ijet], alljetskt4[jjet], RAPIDITY);
          if (myDR < drmin)  drmin = myDR;
        }
      }
      if (drmin <= dRcut)  vetoEvent;

      if (jet1.pt() <= 100*GeV)  vetoEvent;
      if (jet4.pt() <=  64*GeV)  vetoEvent;

      
      float HT = 0; //scalar sum of pt of 4 leading jets
      float Mjjjj = -99; // combined mass of 4 leading jets
      float Mjj = 99999;// minimum combined mass of 2 jets
      float minDphi_ij = 999; // minimum azimuthal distance btw 2 jets
      float minDphi_ijk = 999;// minimum azimuthal distance btw 3 jets
      float maxDrap_ij = -99;// maximum rapidity distance btw 2 jets
      float minDrap_ij = 999;// minimum rapidity distance btw 2 jets
      float minDrap_ijk = 999;// minimum rapidity distance btw 3 jets
      // scalar sum pt of central jets, with different rapidity gaps
      float sumpt_twojets_cent =0;

      //auxiliary variables
      FourMomentum sum_alljets;
      FourMomentum sum_twojets;
      float auxDphi;
      float auxDrap;
      float auxDphi2;
      float auxDrap2;
      int maxY_i = -1;
      int maxY_j = -1;

      //Calculate variables: loop over 4 leading jets
      for (int ij = 0; ij< 4; ++ij) {

        sum_alljets = sum_alljets + leadingJetskt4.at(ij);
        HT += leadingJetskt4.at(ij).pt();

        for (int jj = 0; jj< 4; ++jj) {
          if ( ij == jj )  continue;
          auxDphi = fabs(deltaPhi(leadingJetskt4.at(ij), leadingJetskt4.at(jj)));
          if (auxDphi < minDphi_ij)  minDphi_ij = auxDphi;

          auxDrap = fabs(deltaRap(leadingJetskt4.at(ij), leadingJetskt4.at(jj)));
          if (auxDrap < minDrap_ij)  minDrap_ij = auxDrap;
          if (auxDrap > maxDrap_ij) {
            maxDrap_ij = auxDrap;
            maxY_i = ij;
            maxY_j = jj;
          }

          sum_twojets = leadingJetskt4.at(ij) + leadingJetskt4.at(jj);
          if (sum_twojets.mass() < Mjj)  Mjj = sum_twojets.mass();

          for (int kj = 0; kj < 4; ++kj) {
            if (kj == ij || kj == jj)  continue;
            auxDphi2 = auxDphi + fabs(deltaPhi(leadingJetskt4.at(ij), leadingJetskt4.at(kj)));
            if (auxDphi2 < minDphi_ijk)  minDphi_ijk = auxDphi2;

            auxDrap2 = auxDrap + fabs(deltaRap(leadingJetskt4.at(ij), leadingJetskt4.at(kj)));
            if (auxDrap2 < minDrap_ijk)  minDrap_ijk = auxDrap2;
          }
        }
        
      }//eof loop over leading jets

      Mjjjj = sum_alljets.mass();

      //sum of central jets pt
      for (int ijet = 0; ijet < 4; ++ijet) {
        if (ijet == maxY_i || ijet == maxY_j) {//these are the forward jets
          continue; 
        }
        sumpt_twojets_cent = sumpt_twojets_cent + leadingJetskt4.at(ijet).pt();
      }


      //Fill Histos
      // mass and pT cuts in which the analysis tables are split
      // values are in GeV
      // cuts are inclusive
      float m4jcuts[4]   = {500, 1000, 1500, 2000};
      float pt1cutA[4]   = {100,  400,  700, 1000};
      float pt1cutB[4]   = {100,  250,  400,  550};
      float rapGapCut[4] = {1, 2, 3, 4};

      _h["pt1"]->fill(jet1.pt(), weight);
      _h["pt2"]->fill(jet2.pt(), weight);
      _h["pt3"]->fill(jet3.pt(), weight);
      _h["pt4"]->fill(jet4.pt(), weight);
      _h[ "HT"]->fill(HT,        weight);
      _h["M4j"]->fill(Mjjjj,     weight);

      for (int i_cut = 0; i_cut < 4; ++i_cut) {

        string label = "M2jratio_" + to_str(i_cut);
        if (Mjjjj > m4jcuts[i_cut]) {
          _h[label]->fill( Mjj/Mjjjj , weight);
        }

        if (jet1.pt() > pt1cutA[i_cut]) {
          label = "dPhiMin2j_" + to_str(i_cut);
          _h[label]->fill(minDphi_ij , weight);
          label = "dPhiMin3j_" + to_str(i_cut);
          _h[label]->fill(minDphi_ijk, weight);
          label = "dYMin2j_" + to_str(i_cut);
          _h[label]->fill(minDrap_ij , weight);
          label = "dYMin3j_" + to_str(i_cut);
          _h[label]->fill(minDrap_ijk, weight);
        }
        
        if (jet1.pt() > pt1cutB[i_cut]) {
          label = "dYMax2j_" + to_str(i_cut);
          _h[label]->fill( maxDrap_ij , weight);

          for (int yy = 0; yy < 4; ++yy) {
            if (maxDrap_ij > rapGapCut[yy]) {
              label = "sumPtCent_" + to_str(yy) + to_str(i_cut);
              _h[label]->fill(sumpt_twojets_cent, weight);
            }
          }
        }//eof rapidity cuts

      }//end of loop over pt/m4j cuts

    }//eof analyze


    /// Normalise histograms etc., after the run
    void finalize() {
      const double sf(crossSection() / sumOfWeights());
      for (map<string, Histo1DPtr>::iterator hit = _h.begin(); hit != _h.end(); ++hit) {
        scale(hit->second, sf);
      }
    }

    //@


  private:

    // Data members like post-cuts event weight counters go here

    /// @name Histograms
    //@{
    map<string, Histo1DPtr> _h;
    //@}


  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2015_I1394679);
}
